@@ -84,11 +84,15 @@ def fetch_part_info(part_number: str) -> dict:
     part_info['parameters'] = {}
     [parameter_key, name_key, value_key] = PARAMETERS_MAP
 
-    for parameter in range(len(part[parameter_key])):
-        parameter_name = part[parameter_key][parameter][name_key]
-        parameter_value = part[parameter_key][parameter][value_key]
-        # Append to parameters dictionary
-        part_info['parameters'][parameter_name] = parameter_value
+    try:
+        for parameter in range(len(part[parameter_key])):
+            parameter_name = part[parameter_key][parameter][name_key]
+            parameter_value = part[parameter_key][parameter][value_key]
+            # Append to parameters dictionary
+            part_info['parameters'][parameter_name] = parameter_value
+    except TypeError:
+        # Parameter list is empty
+        pass
 
     return part_info
 
